/*-------------------------------------------------------------------------
 * Copyright (c) 2001-2002 Kenneth W. Sodemann (stufflehead@bigfoot.com)
 *-------------------------------------------------------------------------
 * prop_dlg
 *
 * Synopsis:
 *   Properties Dlg
 *
 * $Id: props.c,v 1.6 2002/01/26 23:36:31 stuffle Exp $
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to
 * Free Software Foundation, Inc.
 * 59 Temple Place, Suite 330 
 * Boston, MA  02111-1307  USA
 *
 *-------------------------------------------------------------------------
 */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>

#include "defs.h"
#include "gtkutils.h"
#include "mainwin.h"
#include "props.h"
#include "util.h"

#define MIN_MINUTES  0
#define MAX_MINUTES  120

/*
 * widgets
 */
#define PROP_DLG                "prop_dlg"
#define AUTO_REF_SPINNER        "auto_ref_spinner"
#define HTML_VIEWER_ENTRY       "html_viewer_entry"
#define TEXT_VIEWER_ENTRY       "text_viewer_entry"
#define SHOW_LOGIN_CB           "show_login_cb"

/*
 * INI Key defines
 */
#define PROP_INI_FILE           "properties"

/* general section */
#define GENERAL                 "general"
#define AUTO_REF_RATE           "auto_refresh_rate"
#define HTML_VIEWER             "html_viewer"
#define DEF_HTML_VIEWER         "gnome-help-browser"
#define TEXT_VIEWER             "text_viwer"
#define DEF_TEXT_VIEWER         "gless"
#define SHOW_LOGIN              "show_login_dlg"

/* Main Window section */
#define MAIN_WIN                       "main_window"
static const char *assignee_str =      "assignee";
static const char *problem_type_str =  "problem_type";
static const char *prj_name_str =      "project_name";
static const char *pr_descr_str =      "problem_rpt_descr";
static const char *pr_rpt_num_str =    "problem_rpt_number";
static const char *severity_str =      "severity";
static const char *status_str =        "status";
static const char *submission_dt_str = "submission_date";
static const char *submitter_str =     "submitter";


static void
on_prop_dlg_destroy (GtkWidget   *widget,
                     gpointer     user_data)
{
   /*
    * Just drop any un-committed changes.
    */
   gnome_config_drop_all();
}


static void
on_apply (GtkWidget *propertybox,
          gint       page_num,
          gpointer   user_data)
{
   GtkWidget *main_win;

   main_win = GTK_WIDGET (gtk_object_get_data (GTK_OBJECT (propertybox),
                                               MY_PARENT));
   g_assert (main_win != NULL);
   if (page_num == -1)
      {
      gnome_config_sync();
      set_auto_refresh_time (main_win);
      }
}


static void
on_help (GtkWidget *propertybox,
         gint       page_num,
         gpointer   user_data)
{
   static GnomeHelpMenuEntry help_ref0 = { "PRepS",
                                           "preps-conf.html#GEN-TAB" };
   static GnomeHelpMenuEntry help_ref1 = { "PRepS",
                                           "preps-conf.html#MAIN-WIN-TAB" };

   switch (page_num)
      {
      case 0:
         gnome_help_display (NULL, &help_ref0);
         break;

      case 1:
         gnome_help_display (NULL, &help_ref1);
         break;

      default:
         g_assert_not_reached();
      }
}


static void
on_auto_ref_spinner_changed (GtkWidget *widget, gpointer user_data)
{
   GString  *str;
   gint      value;

   str = g_string_new ("");
   value = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (widget));
   g_string_sprintf (str, BASIC_CONF, PACKAGE, PROP_INI_FILE,
                     GENERAL, AUTO_REF_RATE);
   gnome_config_set_int (str->str, value);
   g_string_free (str, TRUE);

   gnome_property_box_changed (GNOME_PROPERTY_BOX (user_data));
}


static void
on_html_viewer_entry_changed (GtkWidget *widget, gpointer user_data)
{
   GString  *str;
   gchar    *value;

   str = g_string_new ("");
   value = gtk_entry_get_text (GTK_ENTRY (widget));
   g_string_sprintf (str, BASIC_CONF, PACKAGE, PROP_INI_FILE,
                     GENERAL, HTML_VIEWER);
   gnome_config_set_string (str->str, value);
   g_string_free (str, TRUE);

   gnome_property_box_changed (GNOME_PROPERTY_BOX (user_data));
}


static void
on_text_viewer_entry_changed (GtkWidget *widget, gpointer user_data)
{
   GString  *str;
   gchar    *value;

   str = g_string_new ("");
   value = gtk_entry_get_text (GTK_ENTRY (widget));
   g_string_sprintf (str, BASIC_CONF, PACKAGE, PROP_INI_FILE,
                     GENERAL, TEXT_VIEWER);
   gnome_config_set_string (str->str, value);
   g_string_free (str, TRUE);

   gnome_property_box_changed (GNOME_PROPERTY_BOX (user_data));
}


static void
on_show_login_dlg_button_toggled (GtkWidget *widget, gpointer user_data)
{
   GString  *str;
   gboolean  value;

   str = g_string_new ("");
   value = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget));
   g_string_sprintf (str, BASIC_CONF, PACKAGE, PROP_INI_FILE,
                     GENERAL, SHOW_LOGIN);
   gnome_config_set_bool (str->str, value);
   g_string_free (str, TRUE);

   gnome_property_box_changed (GNOME_PROPERTY_BOX (user_data));
}


static void
on_disp_res_cb_toggled (GtkWidget *widget, gpointer user_data)
{
   GString   *str;
   gboolean   value;
   GtkWidget *prop;

   prop = get_widget (widget, PROP_DLG);
   g_assert (prop != NULL);

   str = g_string_new ("");
   value = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget));
   g_string_sprintf (str, BASIC_CONF, PACKAGE, PROP_INI_FILE,
                     MAIN_WIN, (char *)user_data);
   gnome_config_set_bool (str->str, value);
   g_string_free (str, TRUE);

   gnome_property_box_changed (GNOME_PROPERTY_BOX (prop));
}


gboolean
display_column_in_results (DisplayColumn col)
{
   const gchar *col_str;
   gboolean     def;
   GString     *str;
   gboolean     value;

   switch (col)
      {
      case ASSIGNEE_COL:
         col_str = assignee_str;
         def = FALSE;
         break;

      case PROBLEM_TYPE_COL:
         col_str = problem_type_str;
         def = TRUE;
         break;

      case PRJ_NAME_COL:
         col_str = prj_name_str;
         def = TRUE;
         break;

      case PR_DESCR_COL:
         col_str = pr_descr_str;
         def = TRUE;
         break;

      case PR_RPT_NUM_COL:
         col_str = pr_rpt_num_str;
         def = TRUE;
         break;

      case SEVERITY_COL:
         col_str = severity_str;
         def = TRUE;
         break;

      case STATUS_COL:
         col_str = status_str;
         def = TRUE;
         break;

      case SUBMISSION_DATE_COL:
         col_str = submission_dt_str;
         def = FALSE;
         break;

      case SUBMITTER_COL:
         col_str = submitter_str;
         def = FALSE;
         break;

      default:
         col_str = NULL;
         def = FALSE;
         g_assert_not_reached();
         break;
      }

   str = g_string_new ("");
   g_string_sprintf (str, CONF_WITH_DEF, PACKAGE, PROP_INI_FILE,
                     MAIN_WIN, col_str, def ? "TRUE" : "FALSE");

   value = gnome_config_get_bool (str->str);
   g_string_free (str, TRUE);

   return value;
}


gint
auto_refresh_rate (void)
{
   GString  *str;
   gint      value;
   gboolean  is_def;

   str = g_string_new ("");

   g_string_sprintf (str, CONF_WITH_DEF, PACKAGE, PROP_INI_FILE,
                     GENERAL, AUTO_REF_RATE, "0");
   value = gnome_config_get_int_with_default (str->str, &is_def); 

   g_string_free (str, TRUE);

   return value;
}

gchar *
html_viewer (void)
{
   GString  *str;
   gchar    *buffer;
   gboolean  is_def;

   str = g_string_new ("");
   
   g_string_sprintf (str, CONF_WITH_DEF, PACKAGE, PROP_INI_FILE,
                     GENERAL, HTML_VIEWER, DEF_HTML_VIEWER);
   buffer = gnome_config_get_string_with_default (str->str, &is_def);

   g_string_free (str, TRUE);

   return buffer;
}

gchar *
text_viewer (void)
{
   GString  *str;
   gchar    *buffer;
   gboolean  is_def;

   str = g_string_new ("");
   
   g_string_sprintf (str, CONF_WITH_DEF, PACKAGE, PROP_INI_FILE,
                     GENERAL, TEXT_VIEWER, DEF_TEXT_VIEWER);
   buffer = gnome_config_get_string_with_default (str->str, &is_def);

   g_string_free (str, TRUE);

   return buffer;
}

gboolean
show_login_dlg_on_startup (void)
{
   GString  *str;
   gboolean  flag;
   gboolean  is_def;

   str = g_string_new ("");

   g_string_sprintf (str, CONF_WITH_DEF, PACKAGE, PROP_INI_FILE,
                     GENERAL, SHOW_LOGIN, "false");
   flag = gnome_config_get_bool_with_default (str->str, &is_def);

   g_string_free (str, TRUE);

   return flag;
}


static void
create_general_tab (GtkWidget *prop_box)
{
   GtkObject  *adj;
   GtkWidget  *tab_lbl;
   GtkWidget  *vbox;
   GtkWidget  *hbox;
   GtkWidget  *label;
   GtkWidget  *spinner;
   GtkWidget  *frame;
   GtkWidget  *table;
   GtkWidget  *entry;
   GtkWidget  *cb;
   GtkWidget  *popup;
   gint        value;
   gchar      *buffer;

   tab_lbl = gtk_label_new (_("General"));
   gtk_widget_ref (tab_lbl);
   gtk_object_set_data_full (GTK_OBJECT (prop_box),
                             "general_tab_lbl",
                             tab_lbl,
                            (GtkDestroyNotify) gtk_widget_unref);
   gtk_widget_show (tab_lbl);

   vbox = gtk_vbox_new (FALSE, 3);
   gtk_widget_ref (vbox);
   gtk_object_set_data_full (GTK_OBJECT (prop_box),
                             "general_vbox",
                             vbox,
                             (GtkDestroyNotify) gtk_widget_unref);
   gtk_container_set_border_width (GTK_CONTAINER (vbox), 5);
   gtk_widget_show (vbox);

   hbox = gtk_hbox_new (FALSE, 3);
   gtk_widget_ref (hbox);
   gtk_object_set_data_full (GTK_OBJECT (prop_box),
                             "auto_ref_hbox",
                             hbox,
                             (GtkDestroyNotify) gtk_widget_unref);
   gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
   gtk_widget_show (hbox);

   label = gtk_label_new (_("Auto-Refresh Rate (minutes):"));
   gtk_widget_ref (label);
   gtk_object_set_data_full (GTK_OBJECT (prop_box),
                             "auto_ref_label",
                             label,
                             (GtkDestroyNotify) gtk_widget_unref);
   gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
   gtk_widget_show (label);

   value = auto_refresh_rate();
   adj = gtk_adjustment_new (value, MIN_MINUTES, MAX_MINUTES, 1, 10, 10);
   spinner = gtk_spin_button_new (GTK_ADJUSTMENT (adj), 1, 0);
   gtk_widget_ref (spinner);
   gtk_object_set_data_full (GTK_OBJECT (prop_box),
                             AUTO_REF_SPINNER,
                             spinner,
                             (GtkDestroyNotify) gtk_widget_unref);
   gtk_signal_connect (GTK_OBJECT (spinner), "changed",
                       GTK_SIGNAL_FUNC (on_auto_ref_spinner_changed),
                       prop_box);
   gtk_box_pack_start (GTK_BOX (hbox), spinner, FALSE, FALSE, 0);
   gtk_widget_show (spinner);

   frame = gtk_frame_new (_("Viewers"));
   gtk_widget_ref (frame);
   gtk_object_set_data_full (GTK_OBJECT (prop_box),
                             "viewer_frame",
                             frame,
                             (GtkDestroyNotify) gtk_widget_unref);
   gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, FALSE, 0);
   gtk_widget_show (frame);

   table = gtk_table_new (2, 2, FALSE);
   gtk_widget_ref (table);
   gtk_object_set_data_full (GTK_OBJECT (prop_box),
                             "viewer_table",
                             table,
                             (GtkDestroyNotify) gtk_widget_unref);
   gtk_container_add (GTK_CONTAINER (frame), table);
   gtk_container_set_border_width (GTK_CONTAINER (table), 5);
   gtk_table_set_row_spacings (GTK_TABLE (table), 3);
   gtk_table_set_col_spacings (GTK_TABLE (table), 5);
   gtk_widget_show (table);

   label = gtk_label_new (_("HTML:"));
   gtk_widget_ref (label);
   gtk_object_set_data_full (GTK_OBJECT (prop_box),
                             "html_label",
                             label,
                             (GtkDestroyNotify) gtk_widget_unref);
   gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1,
                     (GtkAttachOptions) GTK_FILL,
                     (GtkAttachOptions) 0, 0, 0);
   gtk_widget_show (label);

   label = gtk_label_new (_("Text:"));
   gtk_widget_ref (label);
   gtk_object_set_data_full (GTK_OBJECT (prop_box),
                             "text_label",
                             label,
                             (GtkDestroyNotify) gtk_widget_unref);
   gtk_table_attach (GTK_TABLE (table), label, 0, 1, 1, 2,
                     (GtkAttachOptions) GTK_FILL,
                     (GtkAttachOptions) 0, 0, 0);
   gtk_widget_show (label);

   entry = gtk_entry_new ();
   gtk_widget_ref (entry);
   gtk_object_set_data_full (GTK_OBJECT (prop_box),
                             HTML_VIEWER_ENTRY,
                             entry,
                             (GtkDestroyNotify) gtk_widget_unref);
   gtk_table_attach (GTK_TABLE (table), entry, 1, 2, 0, 1,
                     (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                     (GtkAttachOptions) 0, 0, 0);
   buffer = html_viewer();
   gtk_entry_set_text (GTK_ENTRY (entry), buffer);
   g_free (buffer);
   gtk_signal_connect (GTK_OBJECT (entry), "changed",
                       GTK_SIGNAL_FUNC (on_html_viewer_entry_changed),
                       prop_box);
   gtk_widget_show (entry);
   popup = attach_generic_edit_popup_menu (entry);
   gtk_widget_ref (popup);
   gtk_object_set_data_full (GTK_OBJECT (prop_box), "popup1", popup,
                             (GtkDestroyNotify) gtk_widget_unref);

   entry = gtk_entry_new ();
   gtk_widget_ref (entry);
   gtk_object_set_data_full (GTK_OBJECT (prop_box),
                             TEXT_VIEWER_ENTRY,
                             entry,
                             (GtkDestroyNotify) gtk_widget_unref);
   gtk_table_attach (GTK_TABLE (table), entry, 1, 2, 1, 2,
                     (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                     (GtkAttachOptions) 0, 0, 0);
   buffer = text_viewer();
   gtk_entry_set_text (GTK_ENTRY (entry), buffer);
   g_free (buffer);
   gtk_signal_connect (GTK_OBJECT (entry), "changed",
                       GTK_SIGNAL_FUNC (on_text_viewer_entry_changed),
                       prop_box);
   gtk_widget_show (entry);
   popup = attach_generic_edit_popup_menu (entry);
   gtk_widget_ref (popup);
   gtk_object_set_data_full (GTK_OBJECT (prop_box), "popup2", popup,
                             (GtkDestroyNotify) gtk_widget_unref);

   cb = gtk_check_button_new_with_label (_("Show login dialog on startup"));
   gtk_widget_ref (cb);
   gtk_object_set_data_full (GTK_OBJECT (prop_box),
                             SHOW_LOGIN_CB,
                             cb,
                             (GtkDestroyNotify) gtk_widget_unref);
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cb),
                                 show_login_dlg_on_startup());
   gtk_box_pack_start (GTK_BOX (vbox), cb, FALSE, FALSE, 0);
   gtk_signal_connect (GTK_OBJECT (cb), "toggled",
                       GTK_SIGNAL_FUNC (on_show_login_dlg_button_toggled),
                       prop_box);
   gtk_widget_show (cb);
   
   gnome_property_box_append_page (GNOME_PROPERTY_BOX (prop_box),
                                   vbox, tab_lbl);
}


static void
create_main_win_tab (GtkWidget *prop_box)
{
   GtkWidget  *tab_lbl;
   GtkWidget  *frame;
   GtkWidget  *table;
   GtkWidget  *cb;
   GtkWidget  *lbl;

   tab_lbl = gtk_label_new (_("Main Window"));
   gtk_widget_ref (tab_lbl);
   gtk_object_set_data_full (GTK_OBJECT (prop_box),
                             "main_win_tab_lbl", tab_lbl,
                             (GtkDestroyNotify) gtk_widget_unref);
   gtk_widget_show (tab_lbl);

   frame = gtk_frame_new (_("Display Columns in Results"));
   gtk_widget_ref (frame);
   gtk_object_set_data_full (GTK_OBJECT (prop_box),
                             "disp_frame", frame,
                             (GtkDestroyNotify) gtk_widget_unref);
   gtk_container_set_border_width (GTK_CONTAINER (frame), 5);
   gtk_widget_show (frame);

   table = gtk_table_new (6, 2, FALSE);
   gtk_widget_ref (table);
   gtk_object_set_data_full (GTK_OBJECT (prop_box),
                             "disp_table", table,
                             (GtkDestroyNotify) gtk_widget_unref);
   gtk_container_add (GTK_CONTAINER (frame), table);
   gtk_widget_show (table);

   lbl = gtk_label_new
      (_("Modifications will take effect the next time PRepS is started."));
   gtk_widget_ref (lbl);
   gtk_object_set_data_full (GTK_OBJECT (prop_box),
                             "note1", lbl,
                             (GtkDestroyNotify) gtk_widget_unref);
   gtk_table_attach (GTK_TABLE (table), lbl, 0, 2, 5, 6,
                     (GtkAttachOptions) (GTK_FILL),
                     (GtkAttachOptions) (0), 0, 0);
   gtk_widget_show (lbl);

   cb = gtk_check_button_new_with_label (_("Problem Report Number"));
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cb),
                                 display_column_in_results (PR_RPT_NUM_COL));
   gtk_widget_ref (cb);
   gtk_object_set_data_full (GTK_OBJECT (prop_box),
                             "pr_num_cb", cb,
                             (GtkDestroyNotify) gtk_widget_unref);
   gtk_signal_connect (GTK_OBJECT (cb), "toggled",
                       GTK_SIGNAL_FUNC (on_disp_res_cb_toggled),
                       (char *)pr_rpt_num_str);
   gtk_table_attach (GTK_TABLE (table), cb, 0, 1, 0, 1,
                     (GtkAttachOptions) (GTK_FILL),
                     (GtkAttachOptions) (0), 0, 0);
   gtk_widget_show (cb);

   cb = gtk_check_button_new_with_label (_("Problem Report Description"));
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cb),
                                 display_column_in_results (PR_DESCR_COL));
   gtk_widget_ref (cb);
   gtk_object_set_data_full (GTK_OBJECT (prop_box),
                             "pr_desc_cb", cb,
                             (GtkDestroyNotify) gtk_widget_unref);
   gtk_signal_connect (GTK_OBJECT (cb), "toggled",
                       GTK_SIGNAL_FUNC (on_disp_res_cb_toggled),
                       (char *)pr_descr_str);
   gtk_table_attach (GTK_TABLE (table), cb, 0, 1, 1, 2,
                     (GtkAttachOptions) (GTK_FILL),
                     (GtkAttachOptions) (0), 0, 0);
   gtk_widget_show (cb);

   cb = gtk_check_button_new_with_label (_("Problem Type"));
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cb),
                                 display_column_in_results (PROBLEM_TYPE_COL));
   gtk_widget_ref (cb);
   gtk_object_set_data_full (GTK_OBJECT (prop_box),
                             "pr_type_cb", cb,
                             (GtkDestroyNotify) gtk_widget_unref);
   gtk_signal_connect (GTK_OBJECT (cb), "toggled",
                       GTK_SIGNAL_FUNC (on_disp_res_cb_toggled),
                       (char *)problem_type_str);
   gtk_table_attach (GTK_TABLE (table), cb, 0, 1, 2, 3,
                     (GtkAttachOptions) (GTK_FILL),
                     (GtkAttachOptions) (0), 0, 0);
   gtk_widget_show (cb);

   cb = gtk_check_button_new_with_label (_("Status"));
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cb),
                                 display_column_in_results (STATUS_COL));
   gtk_widget_ref (cb);
   gtk_object_set_data_full (GTK_OBJECT (prop_box),
                             "status_cb", cb,
                             (GtkDestroyNotify) gtk_widget_unref);
   gtk_signal_connect (GTK_OBJECT (cb), "toggled",
                       GTK_SIGNAL_FUNC (on_disp_res_cb_toggled),
                       (char *)status_str);
   gtk_table_attach (GTK_TABLE (table), cb, 0, 1, 3, 4,
                     (GtkAttachOptions) (GTK_FILL),
                     (GtkAttachOptions) (0), 0, 0);
   gtk_widget_show (cb);

   cb = gtk_check_button_new_with_label (_("Severity"));
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cb),
                                 display_column_in_results (SEVERITY_COL));
   gtk_widget_ref (cb);
   gtk_object_set_data_full (GTK_OBJECT (prop_box),
                             "severity_cb", cb,
                             (GtkDestroyNotify) gtk_widget_unref);
   gtk_signal_connect (GTK_OBJECT (cb), "toggled",
                       GTK_SIGNAL_FUNC (on_disp_res_cb_toggled),
                       (char *)severity_str);
   gtk_table_attach (GTK_TABLE (table), cb, 0, 1, 4, 5,
                     (GtkAttachOptions) (GTK_FILL),
                     (GtkAttachOptions) (0), 0, 0);
   gtk_widget_show (cb);

   cb = gtk_check_button_new_with_label (_("Project Name"));
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cb),
                                 display_column_in_results (PRJ_NAME_COL));
   gtk_widget_ref (cb);
   gtk_object_set_data_full (GTK_OBJECT (prop_box),
                             "prj_name_cb", cb,
                             (GtkDestroyNotify) gtk_widget_unref);
   gtk_signal_connect (GTK_OBJECT (cb), "toggled",
                       GTK_SIGNAL_FUNC (on_disp_res_cb_toggled),
                       (char *)prj_name_str);
   gtk_table_attach (GTK_TABLE (table), cb, 1, 2, 0, 1,
                     (GtkAttachOptions) (GTK_FILL),
                     (GtkAttachOptions) (0), 0, 0);
   gtk_widget_show (cb);

   cb = gtk_check_button_new_with_label (_("Assignee"));
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cb),
                                 display_column_in_results (ASSIGNEE_COL));
   gtk_widget_ref (cb);
   gtk_object_set_data_full (GTK_OBJECT (prop_box),
                             "assignee_cb", cb,
                             (GtkDestroyNotify) gtk_widget_unref);
   gtk_signal_connect (GTK_OBJECT (cb), "toggled",
                       GTK_SIGNAL_FUNC (on_disp_res_cb_toggled),
                       (char *)assignee_str);
   gtk_table_attach (GTK_TABLE (table), cb, 1, 2, 1, 2,
                     (GtkAttachOptions) (GTK_FILL),
                     (GtkAttachOptions) (0), 0, 0);
   gtk_widget_show (cb);

   cb = gtk_check_button_new_with_label (_("Submitter"));
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cb),
                                 display_column_in_results (SUBMITTER_COL));
   gtk_widget_ref (cb);
   gtk_object_set_data_full (GTK_OBJECT (prop_box),
                             "submitter_cb", cb,
                             (GtkDestroyNotify) gtk_widget_unref);
   gtk_signal_connect (GTK_OBJECT (cb), "toggled",
                       GTK_SIGNAL_FUNC (on_disp_res_cb_toggled),
                       (char *)submitter_str);
   gtk_table_attach (GTK_TABLE (table), cb, 1, 2, 2, 3,
                     (GtkAttachOptions) (GTK_FILL),
                     (GtkAttachOptions) (0), 0, 0);
   gtk_widget_show (cb);

   cb = gtk_check_button_new_with_label (_("Submission Date"));
   gtk_toggle_button_set_active
      (GTK_TOGGLE_BUTTON (cb),
       display_column_in_results (SUBMISSION_DATE_COL));
   gtk_widget_ref (cb);
   gtk_object_set_data_full (GTK_OBJECT (prop_box),
                             "sub_dt_cb", cb,
                             (GtkDestroyNotify) gtk_widget_unref);
   gtk_signal_connect (GTK_OBJECT (cb), "toggled",
                       GTK_SIGNAL_FUNC (on_disp_res_cb_toggled),
                       (char *)submission_dt_str);
   gtk_table_attach (GTK_TABLE (table), cb, 1, 2, 3, 4,
                     (GtkAttachOptions) (GTK_FILL),
                     (GtkAttachOptions) (0), 0, 0);
   gtk_widget_show (cb);

   gnome_property_box_append_page (GNOME_PROPERTY_BOX (prop_box),
                                   frame, tab_lbl);
}


static GtkWidget *
create_prop_dlg (GtkWidget *main_win)
{
   GtkWidget *prop_dlg;

   prop_dlg = gnome_property_box_new();
   gtk_object_set_data (GTK_OBJECT (prop_dlg), MY_PARENT, main_win);
   gtk_object_set_data (GTK_OBJECT (prop_dlg), PROP_DLG, prop_dlg);
   gnome_dialog_set_parent (GNOME_DIALOG (prop_dlg), GTK_WINDOW (main_win));

   create_general_tab (prop_dlg);
   create_main_win_tab (prop_dlg);

   return prop_dlg;
}


void
show_prop_dlg (GtkWidget *main_win)
{
   static GtkWidget *prop_dlg = NULL;

   if (prop_dlg)
      {
      gdk_window_show (prop_dlg->window);
      gdk_window_raise (prop_dlg->window);
      }
   else
      {
      prop_dlg = create_prop_dlg (main_win);

      /*
       * When the dialog is destroyed, call gtk_widget_destroyed(), which
       * will set the "user_data" (&prop_dlg in this case), to NULL.
       */
      gtk_signal_connect (GTK_OBJECT (prop_dlg),
                          "destroy",
                          GTK_SIGNAL_FUNC (gtk_widget_destroyed),
                          &prop_dlg);
      gtk_signal_connect (GTK_OBJECT (prop_dlg),
                          "destroy",
                          GTK_SIGNAL_FUNC (on_prop_dlg_destroy),
                          NULL);
      gtk_signal_connect (GTK_OBJECT (prop_dlg),
                          "apply",
                          GTK_SIGNAL_FUNC (on_apply),
                          NULL);
      gtk_signal_connect (GTK_OBJECT (prop_dlg),
                          "help",
                          GTK_SIGNAL_FUNC (on_help),
                          NULL);

      gtk_widget_show (prop_dlg);
      }
}
