#ifndef __PROPS_H
#define __PROPS_H
/*-------------------------------------------------------------------------
 * Copyright (c) 2001-2002 Kenneth W. Sodemann (stufflehead@bigfoot.com)
 *-------------------------------------------------------------------------
 * prop_dlg
 *
 * Synopsis:
 *    Properties Dlg and related functions.
 *
 * $Id: props.h,v 1.2 2002/01/20 02:52:04 stuffle Exp $
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to 
 * Free Software Foundation, Inc. 
 * 59 Temple Place, Suite 330 
 * Boston, MA  02111-1307  USA
 *
 *-------------------------------------------------------------------------
 */

#include <gnome.h>

/*
 * Columns that the user can tell the system to display.
 */
typedef enum {
   ASSIGNEE_COL,
   PROBLEM_TYPE_COL,
   PRJ_NAME_COL,
   PR_DESCR_COL,
   PR_RPT_NUM_COL,
   SEVERITY_COL,
   STATUS_COL,
   SUBMISSION_DATE_COL,
   SUBMITTER_COL
} DisplayColumn;

/*-------------------------------------------------------------------------
 * Function:
 *   display_column_in_results
 *
 * Synopsis:
 *   Determine if the stated column should be displayed on the screen.
 *
 * Inputs:
 *   col : Enumeration (see DislplayColumn) defining the column in
 *         question.
 *
 * Outputs:
 *   None.
 *
 * Return Value:
 *   TRUE  : Display the column
 *   FALSE : Do not display the column
 *-------------------------------------------------------------------------
 */
gboolean display_column_in_results (DisplayColumn col);

/*-------------------------------------------------------------------------
 * Function:
 *   auto_refresh_rate
 *
 * Synopsis:
 *   Return the auto_refresh_rate as defined in the properties.
 *
 * Inputs:
 *   None.
 *
 * Outputs:
 *   None.
 * 
 * Return Value:
 *   The user defined auto refresh rate, in minutes.
 *
 *-------------------------------------------------------------------------
 */
gint auto_refresh_rate (void);

/*-------------------------------------------------------------------------
 * Function:
 *   html_viewer
 *
 * Synopsis:
 *   Return the html viewer as defined in the properties.  It is the
 *   responsibility of the caller to g_free the returned string pointer.
 *
 * Inputs:
 *   None.
 *
 * Outputs:
 *   None.
 * 
 * Return Value:
 *   A pointer to the html viewer as defined by the user.
 *
 *-------------------------------------------------------------------------
 */
gchar *html_viewer (void);

/*-------------------------------------------------------------------------
 * Function:
 *   text_viewer
 *
 * Synopsis:
 *   Return the text viewer as defined in the properties.  It is the
 *   responsibility of the caller to g_free the returned string pointer.
 *
 * Inputs:
 *   None.
 *
 * Outputs:
 *   None.
 * 
 * Return Value:
 *   A pointer to the text viewer as defined by the user.
 *   
 *-------------------------------------------------------------------------
 */
gchar *text_viewer (void);

/*-------------------------------------------------------------------------
 * Function:
 *   show_login_dlg_on_startup
 *
 * Synopsis:
 *   Return a boolean flag that states whether or not the user would
 *   like the login dialog shown on startup instead of attempting to
 *   log on to the default database.
 *
 * Inputs:
 *   None.
 *
 * Outputs:
 *   None.
 * 
 * Return Value:
 *   The value of the flag (TRUE or FALSE) as defined by the user.
 *
 *-------------------------------------------------------------------------
 */
gboolean show_login_dlg_on_startup (void);

/*-------------------------------------------------------------------------
 * Function:
 *   show_prop_dlg
 *
 * Synopsis:
 *   Show the properties dialog box.  The properties dialog is shown
 *   modelessly.  If the box is already being shown, just make sure it
 *   is visable rather than showing it again.
 *
 * Inputs:
 *   main_win : the main window of the application.
 *
 * Outputs:
 *   None.
 * 
 * Return Value:
 *   None.
 *-------------------------------------------------------------------------
 */
void show_prop_dlg (GtkWidget *main_win);

#endif
